==> flutter_map_tile_caching-4.0.0.tar.gz: lib/src/bulk_download/download_progress.dart <==
import 'package:collection/collection.dart';
import 'package:meta/meta.dart';

/// Represents the progress of an ongoing or finished (if [percentageProgress] is 100%) bulk download
///
/// Should avoid manual construction, use named constructor [DownloadProgress.empty] to generate placeholders.
///
/// Is yielded from `StorageCachingTileProvider().downloadRegion()`, and returned in a callback from `StorageCachingTileProvider().downloadRegionBackground()`.
class DownloadProgress {
  /// Number of successful tile downloads
  final int successfulTiles;

  /// List of URLs of failed tiles
  final List<String> failedTiles;

  /// Approximate total number of tiles to be downloaded
  final int maxTiles;

  /// Number of tiles removed because they were entirely sea (these also make up part of [successfulTiles])
  ///
  /// Only applicable if sea tile removal is enabled, otherwise this value is always 0.
  final int seaTiles;

  /// Number of tiles not downloaded because they already existed (these also make up part of [successfulTiles])
  ///
  /// Only applicable if redownload prevention is enabled, otherwise this value is always 0.
  final int existingTiles;

  /// The length of time each tile took to download (one element per tile)
  ///
  /// Use [duration] for time elapsed so far (each element added up)
  final List<Duration> durationPerTile;

  /// Elapsed duration since start of download process
  final Duration duration;

  /// Number of attempted tile downloads, including failures
  ///
  /// Note that this is not used in any other calculations: for example, [remainingTiles] uses [successfulTiles] instead of this.
  ///
  /// Is equal to `successfulTiles + failedTiles.length`.
  int get attemptedTiles => successfulTiles + failedTiles.length;

  /// Approximate number of tiles remaining to be downloaded
  ///
  /// Is equal to `approxMaxTiles - successfulTiles`.
  int get remainingTiles => maxTiles - successfulTiles;

  /// Percentage of tiles saved by using sea tile removal (ie. discount)
  ///
  /// Only applicable if sea tile removal is enabled, otherwise this value is always 0.
  ///
  /// Is equal to `100 - ((((successfulTiles - existingTiles) - seaTiles) / successfulTiles) * 100)`.
  double get seaTilesDiscount => seaTiles == 0
      ? 0
      : 100 -
          ((((successfulTiles - existingTiles) - seaTiles) / successfulTiles) *
              100);

  /// Percentage of tiles saved by using redownload prevention (ie. discount)
  ///
  /// Only applicable if redownload prevention is enabled, otherwise this value is always 0.
  ///
  /// Is equal to `100 -  ((((successfulTiles - seaTiles) - existingTiles) / successfulTiles) * 100)`.
  double get existingTilesDiscount => existingTiles == 0
      ? 0
      : 100 -
          ((((successfulTiles - seaTiles) - existingTiles) / successfulTiles) *
              100);

  /// Approximate percentage of process complete
  ///
  /// Is equal to `(attemptedTiles / approxMaxTiles) * 100`.
  double get percentageProgress => (attemptedTiles / maxTiles) * 100;

  /// Estimated duration for entire download process, based on existing progress and elapsed duration
  ///
  /// Uses an exponentially smoothed moving average algorithm instead of a linear average algorithm. This should lead to more accurate duration calculations, but may not return the same result as expected. The full original algorithm (written in Python) can be found at https://stackoverflow.com/a/54264570/11846040.
  Duration get estTotalDuration {
    final List<int> data =
        durationPerTile.map((e) => e.inMicroseconds).toList();
    const double smoothing = 0.005;

    return Duration(
          microseconds: data.length == 1
              ? data[0]
              : ((smoothing * data.last) +
                      ((1 - smoothing) * (data.sum / data.length)))
                  .round(),
        ) *
        (maxTiles / 2);
  }

  /// Estimated remaining duration until the end of the download process, based on [estTotalDuration]
  ///
  /// Is equal to `estTotalDuration - duration`
  Duration get estRemainingDuration => estTotalDuration - duration;

  /// The 'exponential moving' average duration for download per tile, based on [estTotalDuration].
  ///
  /// Deprecated due to minimal uses, and introduction of more accurate time estimations which do not use linear averaging. No new alternative is provided.
  @Deprecated(
      'Deprecated due to minimal uses, and introduction of more accurate time estimations which do not use linear averaging. No new alternative is provided.')
  Duration get avgDurationTile => Duration(
      microseconds: (estTotalDuration.inMicroseconds / maxTiles).round());

  /// Deprecated due to internal refactoring. Migrate to `attemptedTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.
  @Deprecated(
      'Deprecated due to internal refactoring. Migrate to `attemptedTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.')
  int get completedTiles => attemptedTiles;

  /// Deprecated due to internal refactoring. Migrate to `failedTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.
  @Deprecated(
      'Due to internal refactoring. Migrate to `failedTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.')
  List<String> get erroredTiles => failedTiles;

  /// Deprecated due to internal refactoring. Migrate to `maxTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.
  @Deprecated(
      'Due to internal refactoring. Migrate to `maxTiles` for nearest equivalent. Note that the new alternative is not exactly the same as this: read new documentation for information.')
  int get totalTiles => maxTiles;

  /// Avoid construction using this method. Use [DownloadProgress.empty] to generate empty placeholders where necessary.
  @internal
  DownloadProgress.internal({
    required this.successfulTiles,
    required this.failedTiles,
    required this.maxTiles,
    required this.seaTiles,
    required this.existingTiles,
    required this.durationPerTile,
    required this.duration,
  });

  /// Create an empty placeholder (all values set to 0 or empty) [DownloadProgress], useful for `initalData` in a [StreamBuilder]
  DownloadProgress.empty()
      : successfulTiles = 0,
        failedTiles = [],
        maxTiles = 0,
        seaTiles = 0,
        existingTiles = 0,
        durationPerTile = [],
        duration = const Duration();

  /// Deprecated due to internal refactoring. Migrate to the named constructor [DownloadProgress.empty]. Will be removed in next update.
  @Deprecated(
      'Due to internal refactoring. Migrate to the named constructor [DownloadProgress.empty]. Will be removed in next update.')
  static DownloadProgress get placeholder => DownloadProgress.empty();

  //! GENERAL OBJECT STUFF !//

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is DownloadProgress &&
        other.successfulTiles == successfulTiles &&
        other.failedTiles == failedTiles &&
        other.maxTiles == maxTiles &&
        other.seaTiles == seaTiles &&
        other.existingTiles == existingTiles &&
        other.duration == duration;
  }

  @override
  int get hashCode =>
      successfulTiles.hashCode ^
      failedTiles.hashCode ^
      maxTiles.hashCode ^
      seaTiles.hashCode ^
      existingTiles.hashCode ^
      duration.hashCode;
}

==> flutter_map_tile_caching-5.0.0-dev.1.tar.gz: lib/src/bulk_download/download_progress.dart <==
import 'package:collection/collection.dart';
import 'package:flutter/widgets.dart';
import 'package:meta/meta.dart';

/// Represents the progress of an ongoing or finished (if [percentageProgress] is 100%) bulk download
///
/// Should avoid manual construction, use named constructor [DownloadProgress.empty] to generate placeholders.
///
/// Is yielded from `StorageCachingTileProvider().downloadRegion()`, and returned in a callback from `StorageCachingTileProvider().downloadRegionBackground()`.
class DownloadProgress {
  /// Identification number of the corresponding download
  ///
  /// A zero identification denotes that there is no corresponding download yet, usually due to the initialisation with [DownloadProgress.empty].
  final int downloadID;

  /// Number of successful tile downloads
  final int successfulTiles;

  /// List of URLs of failed tiles
  final List<String> failedTiles;

  /// Approximate total number of tiles to be downloaded
  final int maxTiles;

  /// Number of tiles removed because they were entirely sea (these also make up part of [successfulTiles])
  ///
  /// Only applicable if sea tile removal is enabled, otherwise this value is always 0.
  final int seaTiles;

  /// Number of tiles not downloaded because they already existed (these also make up part of [successfulTiles])
  ///
  /// Only applicable if redownload prevention is enabled, otherwise this value is always 0.
  final int existingTiles;

  /// The length of time each tile took to download (one element per tile)
  ///
  /// Use [duration] for time elapsed so far (each element added up)
  final List<Duration> durationPerTile;

  /// Elapsed duration since start of download process
  final Duration duration;

  /// Number of attempted tile downloads, including failures
  ///
  /// Note that this is not used in any other calculations: for example, [remainingTiles] uses [successfulTiles] instead of this.
  ///
  /// Is equal to `successfulTiles + failedTiles.length`.
  int get attemptedTiles => successfulTiles + failedTiles.length;

  /// Approximate number of tiles remaining to be downloaded
  ///
  /// Is equal to `approxMaxTiles - successfulTiles`.
  int get remainingTiles => maxTiles - successfulTiles;

  /// Percentage of tiles saved by using sea tile removal (ie. discount)
  ///
  /// Only applicable if sea tile removal is enabled, otherwise this value is always 0.
  ///
  /// Is equal to `100 - ((((successfulTiles - existingTiles) - seaTiles) / successfulTiles) * 100)`.
  double get seaTilesDiscount => seaTiles == 0
      ? 0
      : 100 -
          ((((successfulTiles - existingTiles) - seaTiles) / successfulTiles) *
              100);

  /// Percentage of tiles saved by using redownload prevention (ie. discount)
  ///
  /// Only applicable if redownload prevention is enabled, otherwise this value is always 0.
  ///
  /// Is equal to `100 -  ((((successfulTiles - seaTiles) - existingTiles) / successfulTiles) * 100)`.
  double get existingTilesDiscount => existingTiles == 0
      ? 0
      : 100 -
          ((((successfulTiles - seaTiles) - existingTiles) / successfulTiles) *
              100);

  /// Approximate percentage of process complete
  ///
  /// Is equal to `(attemptedTiles / approxMaxTiles) * 100`.
  double get percentageProgress => (attemptedTiles / maxTiles) * 100;

  /// Estimated duration for entire download process, based on existing progress and elapsed duration
  ///
  /// Uses an exponentially smoothed moving average algorithm instead of a linear average algorithm. This should lead to more accurate duration calculations, but may not return the same result as expected. The full original algorithm (written in Python) can be found at https://stackoverflow.com/a/54264570/11846040.
  Duration get estTotalDuration {
    final List<int> data =
        durationPerTile.map((e) => e.inMicroseconds).toList();
    const double smoothing = 0.005;

    return Duration(
          microseconds: data.length == 1
              ? data[0]
              : ((smoothing * data.last) +
                      ((1 - smoothing) * (data.sum / data.length)))
                  .round(),
        ) *
        (maxTiles / 2);
  }

  /// Estimated remaining duration until the end of the download process, based on [estTotalDuration]
  ///
  /// Is equal to `estTotalDuration - duration`
  Duration get estRemainingDuration => estTotalDuration - duration;

  /// Avoid construction using this method. Use [DownloadProgress.empty] to generate empty placeholders where necessary.
  @internal
  DownloadProgress.internal({
    required this.downloadID,
    required this.successfulTiles,
    required this.failedTiles,
    required this.maxTiles,
    required this.seaTiles,
    required this.existingTiles,
    required this.durationPerTile,
    required this.duration,
  });

  /// Create an empty placeholder (all values set to 0 or empty) [DownloadProgress], useful for `initalData` in a [StreamBuilder]
  DownloadProgress.empty()
      : downloadID = 0,
        successfulTiles = 0,
        failedTiles = [],
        maxTiles = 0,
        seaTiles = 0,
        existingTiles = 0,
        durationPerTile = [],
        duration = Duration.zero;

  //! GENERAL OBJECT STUFF !//

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is DownloadProgress &&
        other.successfulTiles == successfulTiles &&
        other.failedTiles == failedTiles &&
        other.maxTiles == maxTiles &&
        other.seaTiles == seaTiles &&
        other.existingTiles == existingTiles &&
        other.duration == duration;
  }

  @override
  int get hashCode =>
      successfulTiles.hashCode ^
      failedTiles.hashCode ^
      maxTiles.hashCode ^
      seaTiles.hashCode ^
      existingTiles.hashCode ^
      duration.hashCode;
}
